﻿# 03.Dec 2020: Row 94: Replace ID of test user (f2637e45-0c9d-406d-b1b8-7149ffa70e8e) to variable $UsernameID

[CmdletBinding(DefaultParameterSetName='default')]
    param
    (
        [Parameter(ParameterSetName='Channel')]
        $Team,
        
        [Parameter(Mandatory=$false,ParameterSetName='default')]
        [Parameter(Mandatory=$true,ParameterSetName='Channel')]
        $Channel

        
    )
Write-Host "
'########:::'######:::'######:::'##::::'##:'##:::'##::::::'########:'##::::'##:
 ##.... ##:'##... ##:'##... ##:: ##:::: ##:. ##:'##::::::: ##.....:: ##:::: ##:
 ##:::: ##: ##:::..:: ##:::..::: ##:::: ##::. ####:::::::: ##::::::: ##:::: ##:
 ########::. ######:: ##::'####: ##:::: ##:::. ##::::::::: ######::: ##:::: ##:
 ##.....::::..... ##: ##::: ##:: ##:::: ##:::: ##::::::::: ##...:::: ##:::: ##:
 ##::::::::'##::: ##: ##::: ##:: ##:::: ##:::: ##::::'###: ##::::::: ##:::: ##:
 ##::::::::. ######::. ######:::. #######::::: ##:::: ###: ########:. #######::
..::::::::::......::::......:::::.......::::::..:::::...::........:::.......:::
"
 



$scriptpath = $MyInvocation.MyCommand.Path
$dir = Split-Path $scriptpath
$Date = Get-Date -Format "MM-dd-yyyy-HHmm"


$clientId = "client-id"
$tenantName = "tenant.onmicrosoft.com"
$clientSecret = "your secret key"
$resource = "https://graph.microsoft.com/"
$Username = "login@tenant.onmicrosoft.com"
$Password = "your-password"
$UsernameID = "objectid_of_username"

$ReqTokenBody = @{
    Grant_Type    = "Password"
    client_Id     = $clientID
    Client_Secret = $clientSecret
    Username      = $Username
    Password      = $Password
    Scope         = "https://graph.microsoft.com/.default"
} 
$TokenResponse = Invoke-RestMethod -Uri "https://login.microsoftonline.com/$TenantName/oauth2/v2.0/token" -Method POST -Body $ReqTokenBody



#Getting all Groups
$apiUrl = "https://graph.microsoft.com/beta/groups/"
$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Get
$Groups = ($Data | Select-Object Value).Value
if ($Team -eq $NULL){
Write-Host "You have" -NoNewline 
Write-Host " $($Groups.Count)" -ForegroundColor Yellow -NoNewline
Write-Host " teams."
Write-Host ""
Write-Host "Messages from which Team do you want to export to the HTML format?" -ForegroundColor Yellow
$Groups | FT DisplayName,Description
$Team = Read-Host "Type one of the Team (DisplayName)"
}
$TeamID = ($Groups | Where-Object {$_.displayname -eq "$($Team)"}).id

$apiUrl = "https://graph.microsoft.com/v1.0/teams/$TeamID/Channels"
$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Get
if ($Channel -eq $NULL){
Write-Host "You choose" -NoNewline
Write-Host " $($Team)" -ForegroundColor Yellow -NoNewline
Write-Host " Team."
Write-Host ""

$Channels = ($Data | Select-Object Value).Value
Write-Host "Messages from which Channel do you want to export to the HTML format?" -ForegroundColor Yellow
$Channels | FT DisplayName,Description
$Channel = Read-Host "Type one of the Channel(DisplayName)"
}

$ChannelID = (($Data | Select-Object Value).Value | Where-Object {$_.displayName -eq "$($Channel)"}).ID

$apiUrl = "https://graph.microsoft.com/beta/groups/$TeamID/members"
$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Get
$UsersIDs = ($Data | Select-Object Value).Value.ID 

# Join $UsernameID to $TeamID if it is not a member
if ($UsersIDs -notcontains $UsernameID){
$apiUrl = "https://graph.microsoft.com/beta/groups/$TeamID/members/`$ref"
$body = @"
{
    "@odata.id": "https://graph.microsoft.com/beta/directoryObjects/$UsernameID"
}
"@
$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Post -ContentType 'application/json' -Body $body
Start-Sleep -Seconds 3
} 
      
       
        
#messages from channel

$apiUrl = "https://graph.microsoft.com/beta/teams/$TeamID/channels/$ChannelID/messages"
$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Get 
$Messages = ($Data | Select-Object Value).Value
$mess = $Messages | Select-Object @{Name = 'DateTime'; Expression = {Get-Date -Date (($_).createdDateTime) -Format 'dd/MM/yyyy HH:mm'}}, @{Name = 'From'; Expression = {((($_).from).user).displayName}}, @{Name = 'Message'; Expression = {(($_).body).content -replace '<.*?>',''}} | Sort-Object DateTime



$Header = @"
<style>
h1, h5, th { text-align: center; } 
table { margin: auto; font-family: Segoe UI; box-shadow: 10px 10px 5px #888; border: thin ridge grey; } 
th { background: #0046c3; color: #fff; max-width: 400px; padding: 5px 10px; } 
td { font-size: 11px; padding: 5px 20px; color: #000; } 
tr { background: #b8d1f3; } 
tr:nth-child(even) { background: #dae5f4; } 
tr:nth-child(odd) { background: #b8d1f3; }
</style>
"@

$body = "<body><b>Generated:</b> $(Get-Date) <br><br> <b>Team Name:</b> $($Team) <br> <b>Channel Name:</b> $($Channel) <br><br>"
$body = $body + "</head>"

$messhtml = $Mess | ConvertTo-Html -Body $body -Head $Header
$Export = "$dir\TeamsHistory\$Team-$Channel"
New-Item -ItemType Directory -Path $Export -ErrorAction Ignore
$messhtml | Out-File $Export\$Team-$Channel-$Date.html

# Remove $UsernameID if it should not be a member of $TeamID
if ($UsersIDs -notcontains $UsernameID){
$apiUrl = "https://graph.microsoft.com/beta/groups/$TeamID/members/$UsernameID/`$ref"

$Data = Invoke-RestMethod -Headers @{Authorization = "Bearer $($Tokenresponse.access_token)"} -Uri $apiUrl -Method Delete
}
 
Write-Host "

"
Write-Host "Messages from the" -NoNewline
Write-Host " $($Team)" -NoNewline -ForegroundColor Yellow
Write-Host " team and" -NoNewline
Write-Host " $($Channel)" -NoNewline -ForegroundColor Yellow
Write-Host " channel were generated and saved to the" -NoNewline
Write-Host " $($Export)" -NoNewline -ForegroundColor Yellow
Write-Host " as a" -NoNewline
Write-Host " $($Team)-$($Channel)-$($Date).html" -NoNewline -ForegroundColor Yellow
Write-Host " file."
Write-Host "

"
